var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import path from 'path';
import { fileURLToPath } from 'url';
import babel from '@babel/core';
const SUPPORTED_EXTENSIONS = ['.js', '.jsx', '.ts', '.tsx'];
/**
 * @param {string} specifier
 * @param {{
 *   conditions: !Array<string>,
 *   parentURL: !(string | undefined),
 * }} context
 * @param {Function} nextResolve
 * @returns {Promise<{ url: string }>}
 */
export function resolve(specifier, context, nextResolve) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const x = yield nextResolve(specifier, context);
            return x;
        }
        catch ( /**@type {any} */error) {
            if (!specifier.startsWith('.') && !specifier.startsWith('/'))
                throw error;
            const extension = path.extname(fileURLToPath(/**@type {import('url').URL}*/ (new URL(specifier, context.parentURL))));
            if (error.code === 'ERR_MODULE_NOT_FOUND' && extension === '.js') {
                const specifierUrl = new URL(specifier, context.parentURL);
                const sameFileButTs = replaceExtension(specifierUrl, '.js', '.ts');
                const resolvedTs = yield tryResolve(sameFileButTs);
                if (resolvedTs)
                    return resolvedTs;
                const sameFileButTsx = replaceExtension(specifierUrl, '.js', '.tsx');
                const resolvedTsx = yield tryResolve(sameFileButTsx);
                if (resolvedTsx)
                    return resolvedTsx;
                throw error;
            }
            else {
                throw error;
            }
        }
        /**
         * @param {string} specifier
         */
        function tryResolve(specifier) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    return yield nextResolve(specifier, context);
                }
                catch (error) {
                    return undefined;
                }
            });
        }
    });
}
/**
 * @param {string} url
 * @param {Object} context
 * @param {Function} defaultGetFormat
 * @returns {Promise<{ format: string }>}
 */
export function getFormat(url, context, defaultGetFormat) {
    return __awaiter(this, void 0, void 0, function* () {
        const urlUrl = new URL(url);
        if (urlUrl.protocol === 'file:') {
            const extension = path.extname(fileURLToPath(url)) || '.js';
            if (SUPPORTED_EXTENSIONS.includes(extension)) {
                return defaultGetFormat(replaceExtension(urlUrl, extension, '.js'), context, defaultGetFormat);
            }
        }
        return defaultGetFormat(url, context, defaultGetFormat);
    });
}
/**
 *
 * @param {URL} url
 * @param {string} fromExtension
 * @param {string} toExtension
 */
function replaceExtension(url, fromExtension, toExtension) {
    url.pathname = url.pathname.slice(0, -fromExtension.length) + toExtension;
    return url.href;
}
/**
 * @param {!(string | SharedArrayBuffer | Uint8Array)} source
 * @param {{
 *   format: string,
 *   url: string,
 * }} context
 * @param {Function} [defaultTransformSource]
 * @returns {Promise<{ source: !(string | SharedArrayBuffer | Uint8Array) }>}
 */
export function transformSource(source, context, defaultTransformSource) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const { url, format } = context;
        if (url.match(/\/node_modules\//) || (format !== 'module' && format !== 'commonjs')) {
            if (defaultTransformSource) {
                return defaultTransformSource(source, context, defaultTransformSource);
            }
            else {
                return { source };
            }
        }
        const stringSource = typeof source === 'string'
            ? source
            : Buffer.isBuffer(source)
                ? source.toString('utf-8')
                : Buffer.from(source).toString('utf-8');
        const sourceCode = (_a = (yield babel.transformAsync(stringSource, {
            sourceType: 'module',
            filename: fileURLToPath(url),
        }))) === null || _a === void 0 ? void 0 : _a.code;
        return sourceCode
            ? {
                source: sourceCode,
            }
            : defaultTransformSource === null || defaultTransformSource === void 0 ? void 0 : defaultTransformSource(source, context, defaultTransformSource);
    });
}
/**
 * @param {string} url
 * @param {{
 *   format: string,
 * }} context
 * @param {Function} nextLoad
 * @returns {Promise<{ source: !(string | SharedArrayBuffer | Uint8Array), format: string}>}
 */
export function load(url, context, nextLoad) {
    return __awaiter(this, void 0, void 0, function* () {
        const { format, source } = yield nextLoad(url, context).catch((/** @type {any} */ error) => __awaiter(this, void 0, void 0, function* () {
            if (error.code === 'ERR_UNKNOWN_FILE_EXTENSION') {
                return yield nextLoad(url, Object.assign(Object.assign({}, context), { format: 'commonjs' }));
            }
            else {
                throw error;
            }
        }));
        if (!url.match(/\/node_modules\//) && source) {
            const transformed = yield transformSource(source, { format, url }, undefined);
            if (transformed) {
                return { source: transformed.source, format };
            }
            else {
                return { format, source };
            }
        }
        else {
            return { format, source };
        }
    });
}
//# sourceMappingURL=babel-register-esm.js.map